/*
 *  dirent.c
 *			Additions to Pure-C for UN*X compatibility
 *
 *  11/19/99  Bodo Wenzel  "opendir()", "readdir()", "rewinddir()",
 *                         and "closedir()"
 */

#include <errno.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include "dirent.h"

DIR *opendir (const char *name) {
/*-----------------------------*/
DIR  *dir;

dir=(DIR *)malloc(sizeof(DIR));
if (dir==NULL) {
  errno=ENOMEM;
  return NULL;
  }

dir->__d_dirname=(char *)malloc(strlen(name)+4);
if (dir->__d_dirname==NULL) {
  free(dir);
  errno=ENOMEM;
  return NULL;
  }
strcpy(dir->__d_dirname,name);
strcat(dir->__d_dirname,"\\*.*");

if (findfirst(dir->__d_dirname,&(dir->__d_ffblk),0)!=0) {
  free(dir->__d_dirname);
  free(dir);
  errno=ENOENT;
  return NULL;
  }

dir->__d_find_first_called=0;
return dir;
}

struct dirent *readdir (DIR *dir) {
/*-------------------------------*/
int  res;

if (dir==NULL) {
  errno=EBADF;
  return NULL;
  }

if (!dir->__d_find_first_called) {
  res=findfirst(dir->__d_dirname,&(dir->__d_ffblk),0);
  dir->__d_find_first_called=1;
} else
  res=findnext(&(dir->__d_ffblk));

if (res!=0) {
  errno=ENOENT;
  return NULL;
  }

strcpy(dir->__d_dirent.d_name,dir->__d_ffblk.ff_name);
return &(dir->__d_dirent);
}

void rewinddir (DIR *dir) {
/*-----------------------*/
if (dir==NULL) {
  errno=EBADF;
  return;
  }

dir->__d_find_first_called=0;
}

int closedir (DIR *dir) {
/*---------------------*/
if (dir==NULL) {
  errno=EBADF;
  return -1;
  }

free(dir->__d_dirname);
free(dir);
return 0;
}
